#!/usr/bin/python

# 	Author: Steven HK Wong
#	  Create Date: Sep 2014
#		Description: A useful logging class 


import sys, getopt
import os
from time import gmtime, localtime, strftime
import datetime

PROGNAME=os.path.basename(sys.argv[0])
LOGFILE = '/dev/stdout'
NONE = int('00000000',2)
INFO = int('00000001',2)
TRACE = int('00000010',2)
DEBUG_1 = int('00000100',2)
DEBUG_2 = int('00001100',2)
DEBUG_3 = int('00011100',2)
LEVEL_STR = { INFO:'info', TRACE:'trace', DEBUG_1:'debug1', DEBUG_2:'debug2', DEBUG_3:'debug3' }

API =  '\tlogger.init([file])\n\
\tlogger.status()\n\
\tlogger.usage()\n\
\tlogger.setNone()\n\
\tlogger.setInfo()\n\
\tlogger.setTrace()\n\
\tlogger.setDebug(level)\n\
\tlogger.setClass(class_list) - list may contain objects or names of classes\n\
\tlogger.log(level,msg)\n\
\tlogger.logc(class,level,msg)\n\
\tlogger.info(msg)\n\
\tlogger.infoc(class,msg)\n\
\tlogger.trace(msg)\n\
\tlogger.tracec(class,msg)\n\
\tlogger.debug(level,msg)\n\
\tlogger.debug1(msg)\n\
\tlogger.debug2(msg)\n\
\tlogger.debug3(msg)\n\
\tlogger.debugc(class,level,msg)\n\
\tlogger.debug1c(class,msg)\n\
\tlogger.debug2c(class,msg)\n\
\tlogger.debug3c(class,msg)\n'

classes = []
log_level = INFO
logfile = LOGFILE

fh = None

def init (lfile=''):
  global logfile
  global fh

  if lfile != '':
    logfile=lfile

  try:
    fh = open(logfile,'a')
  except IOError , e :
    print 'Error:' , e
    sys.exit(2)
  
  
def usage (prefix='', fileHandle=None):
	if fileHandle == None:
		print '%sUsage:%s [-f logfile|--file logfile] [-h]' % (prefix,PROGNAME)
		print ' - logfile defaults to stdout'
	else:
		fileHandle.write ( '%sUsage:%s [-f logfile|--file logfile] [-h]\n' % (prefix,PROGNAME) )
		fileHandle.write ( ' - logfile defaults to stdout\n' )
	status_api(fileHandle)


### Provides classes logging
#-----------------------------------------
def logc (theclass, param1, param2 = None):
  if not hasClass(theclass):
    return

  if param2 == None:
    msg = param1
    level = None
  else:
    level = param1
    msg = param2
  
  ts = strftime("%Y-%m-%d %H:%M:%S", localtime())
  if level == None:
    fh.write( '%s;%s;%s\n' % (ts, theclass.__class__.__name__, msg))
    fh.flush()
  else:
    if ok(level):
      fh.write( '%s;%s;%s;%s\n' % (ts, theclass.__class__.__name__, LEVEL_STR[level], msg))
      fh.flush()

def infoc(theclass,msg):
  logc (theclass,INFO,msg)

def tracec(theclass,msg):
  logc (theclass,TRACE,msg)

def debugc(theclass,param1,param2=None):
  if param2 == None:
    msg = param1
    level = DEBUG1
  else:
    msg = param2
    level = param1
  logc (theclass,level,msg)

def debug1c(theclass,msg):
  logc (theclass,DEBUG_1,msg)

def debug2c(theclass,msg):
  logc (theclass,DEBUG_2,msg)

def debug3c(theclass,msg):
  logc (theclass,DEBUG_3,msg)

def setClass (theclasses):
  global classes
  if theclasses != None:
    for c in theclasses:
			if isinstance(c,str):
				classes.append(c)
			else:
				classes.append(c.__class__.__name__)

def hasClass (theclass):
  if theclass != None:
    if theclass.__class__.__name__ in classes:
      return True;

  return False

### Provides plain logging
#-----------------------------------------
def log (param1, param2 = None):
  if param2 == None:
    msg = param1
    level = None
  else:
    level = param1
    msg = param2
  
  ts = strftime("%Y-%m-%d %H:%M:%S", localtime())
  if level == None:
    fh.write( '%s;%s\n' % (ts, msg))
    fh.flush()
  else:
    if ok(level):
      fh.write( '%s;%s;%s\n' % (ts, LEVEL_STR[level], msg))
      fh.flush()

def info(msg):
  log (INFO,msg)

def trace(msg):
  log (TRACE,msg)

def debug(param1,param2=None):
  # checks single param but single param has to be msg
  if param2 == None:
    level = DEBUG_1
    msg = param1
  else:
    level = param1
    msg = param2
  log (level,msg)

def debug1(msg):
  log (DEBUG_1,msg)

def debug2(msg):
  log (DEBUG_2,msg)

def debug3(msg):
  log (DEBUG_3,msg)

def quiet():
  global log_level 
  log_level = NONE

def setNone():
  quiet()

def setInfo(on=True):
  global log_level 
  if on:
    log_level = log_level | INFO
  else:
    log_level = log_level & ~INFO

def setTrace(on=True):
  global log_level 
  if on:
    log_level = log_level | TRACE
  else:
    log_level = log_level & ~TRACE

def setDebug(level=DEBUG_1):
  global log_level 
  log_level = log_level | level

def ok (level):
  level = int(level)
  if level & log_level == level:
    return True
  else: 
    return False

def status_api(fh=None):
	if fh != None:
		fh.write ('\tAPI: \n')
		fh.write(API)
	else:
		print  '\tAPI: '
		print '%s' % (API)

def status_usage():
	usage('',fh)

def status():
  fh.write ('logfile: %s\n' % (logfile))
  status_usage ()

def main(argv):
  lfile = ''
  try:
    opts, args = getopt.getopt(argv,"hf:",["file="])
  except getopt.GetoptError:
    usage('Error:')
    sys.exit(2)
  for opt, arg in opts:
    if opt == '-h':
      usage()
      sys.exit()
    elif opt in ("-f", "--file"):
      lfile = arg
  init (lfile)

  print 'Testing ..'
  print 'logfile is ', logfile
  if len(args) == 0:
    args = ['Test line']
  
  log ("\n")
  status ()

  log ("quiet level:")
  quiet()
  for arg in args:
    log (INFO,arg)
    log (TRACE,arg)
    debug (arg)
    log (DEBUG_3,arg)

  log ("\n")
  log ("info level:")
  setInfo()
  for arg in args:
    log (INFO,arg)
    log (TRACE,arg)
    debug (arg)
    log (DEBUG_2,arg)
    log (DEBUG_3,arg)
  
  log ("\n")
  log ("trace level:")
  setNone()
  setTrace()
  for arg in args:
    log (INFO,arg)
    log (TRACE,arg)
    debug (arg)
    log (DEBUG_2,arg)
    log (DEBUG_3,arg)

  log ("\n")
  log ("debug-1 level:")
  setNone()
  setDebug(DEBUG_1)
  for arg in args:
    log (INFO,arg)
    log (TRACE,arg)
    debug (arg)
    log (DEBUG_2,arg)
    log (DEBUG_3,arg)

  log ("\n")
  log ("debug-2 level:")
  setNone()
  setDebug(DEBUG_2)
  for arg in args:
    log (INFO,arg)
    log (TRACE,arg)
    debug (arg)
    log (DEBUG_2,arg)
    log (DEBUG_3,arg)

  log ("\n")
  log ("debug-3 + trace level:")
  setNone()
  setTrace()
  setDebug(DEBUG_3)
  for arg in args:
    log (INFO,arg)
    log (TRACE,arg)
    debug (arg)
    log (DEBUG_2,arg)
    log (DEBUG_3,arg)
  
  log ("\n")
  log ("Class A info level:")
  class A(object):
    def myprint(self, msg):
      logc (self, INFO, msg)
      logc (self, TRACE, msg)
      logc (self, DEBUG_1, msg)
      logc (self, DEBUG_2, msg)
      logc (self, DEBUG_3, msg)
  class B(object):
    def myprint(self, msg):
      logc (self, INFO, msg)
      logc (self, TRACE, msg)
      logc (self, DEBUG_1, msg)
      logc (self, DEBUG_2, msg)
      logc (self, DEBUG_3, msg)

  setClass ( [A()] )
  setNone()
  setInfo()
  a = A()
  b = B()
  for arg in args:
    a.myprint(arg)
    b.myprint(arg)
  
  log ("\n")
  log ("Class A and B info level:")
  setClass ( [A(), B()] )
  setNone()
  setInfo()
  for arg in args:
    a.myprint(arg)
    b.myprint(arg)

  log ("\n")
  log ("Class A and B info off , trace on:")
  setInfo(False)
  setTrace()
  for arg in args:
    a.myprint(arg)
    b.myprint(arg)


if __name__ == "__main__":
  # get argv[1] onwards 
  main(sys.argv[1:])




from threading import Thread
from time import gmtime, localtime, strftime
import time
import sys
import os

# directory containing customed modules:
#sys.path.insert(0,"/apps/lib/python")
import logger

COMMAND_SHELL = "python -m aws-cmd -m s3 "
bucket = ''
prefix = ''



class S3Commands():
    
    def __init__(self):
        logger.tracec(self,'S3Commands()')

    def process (self,cmd="", inbucket='', cwd = '', args = []):
        logger.tracec(self,'process(' + cmd + ',bucket=' + inbucket + ', cwd=' +cwd + ')')

        global bucket
        global prefix
        prefix = cwd

        bucket = inbucket
        if cmd == "ls":
          self.ls(args)
          return
        if cmd == "tree":
          self.tree(args)
          return
        if cmd == "get":
          self.get(args)
          return
        if cmd == "put":
          self.put(args)
          return
        if cmd == "rm":
          self.rm(args)
          return
        if cmd == "del":
          try:
            ans = raw_input('Confirm [N/y] ? ')
          except KeyboardInterrupt:
            return
          if ans == 'y':
            self.delete(args)
          return
        print "not implemented"

    def ls (self,args=''):
        logger.tracec(self, '  ls()')
        if bucket:
          thecmd = COMMAND_SHELL + ' -b ' + bucket  + ' -C ls '
        else:
          thecmd = COMMAND_SHELL + ' -C ls'
        if len(args) == 1 and prefix:
          runcmd = thecmd + ' -P ' + prefix + '/' 
          logger.tracec(self, '  running ' + runcmd)
          f = os.popen(runcmd)
          print '%s\n' % (f.read())
        else:
          for arg in args[1:]:
            if prefix:
              runcmd = thecmd + ' -P ' + prefix + '/' + arg
            else:
              runcmd = thecmd + ' -P ' + arg
            logger.tracec(self, '  running ' + runcmd)
            f = os.popen(runcmd)
            print '%s\n' % (f.read())

    def tree (self,args):
        logger.tracec(self, '  tree()')
        if bucket:
          thecmd = COMMAND_SHELL + ' -b ' + bucket  + ' -C tree '
        else:
          thecmd = COMMAND_SHELL + ' -C tree'
        if len(args) == 1:
          if prefix:
            thecmd = thecmd + ' -P ' + prefix + '/' 
          logger.tracec(self, '  running ' + thecmd)
          f = os.popen(thecmd)
          print '%s\n' % (f.read())
        else:
          for arg in args[1:]:
            if prefix:
              runcmd = thecmd + ' -P ' + prefix + '/' + arg
            else:
              runcmd = thecmd + ' -P ' + arg
            logger.tracec(self, '  running ' + runcmd)
            f = os.popen(runcmd)
            print '%s\n' % (f.read())

    def get (self,args):
        logger.tracec(self, '  get()')
        if bucket:
          thecmd = COMMAND_SHELL + ' -b ' + bucket  + ' -C get '
        else:
          thecmd = COMMAND_SHELL + ' -C tree'
        if len(args) != 3:
          print 'Error: Insufficient arguments!'
          logger.tracec(self, '  Error: Insuficient arguments ' )
          return
        if prefix:
          if args[1].startswith('/'):
            _src = args[1]
          else:
            _src = prefix + '/' + args[1]
        else:
            _src = args[1]
        _dest = args[2]
        thecmd = thecmd + ' -P ' + _src + ' -F ' + _dest
        logger.tracec(self, '  running ' + thecmd)
        f = os.popen(thecmd)
        print '%s\n' % (f.read())

    def put (self,args):
        logger.tracec(self, '  put()')
        if bucket:
          thecmd = COMMAND_SHELL + ' -b ' + bucket  + ' -C put '
        else:
          thecmd = COMMAND_SHELL + ' -C tree'
        if len(args) != 3:
          print 'Error: Insufficient arguments!'
          logger.tracec(self, '  Error: Insuficient arguments ' )
          return
        if prefix:
          if args[2].startswith('/'):
            _dest = args[2]
          else:
            _dest = prefix + '/' + args[1]
        else:
            _dest = args[2]
        _src = args[1]
        thecmd = thecmd + ' -F ' + _src + ' -P ' + _dest
        logger.tracec(self, '  running ' + thecmd)
        f = os.popen(thecmd)
        print '%s\n' % (f.read())

    def rm (self,args):
        logger.tracec(self, '  rm()')
        if bucket:
          thecmd = COMMAND_SHELL + ' -b ' + bucket  + ' -C rm '
        else:
          thecmd = COMMAND_SHELL + ' -C tree'
        if len(args) < 2:
          print 'Error: Insufficient arguments!'
          logger.tracec(self, '  Error: Insuficient arguments ' )
          return
        for o in args[1:]:
          if prefix:
            rmcmd = thecmd + ' -P ' + prefix + '/' + o
          else:
            rmcmd = thecmd + ' -P ' + o
          logger.tracec(self, '  running ' + rmcmd)
          f = os.popen(rmcmd)
          print '%s\n' % (f.read())

    def delete (self,args):
        logger.tracec(self, '  del()')
        if bucket:
          thecmd = COMMAND_SHELL + ' -C del -f -b '
        else:
          thecmd = COMMAND_SHELL + ' -C tree'
        if len(args) < 2:
          print 'Error: Insufficient arguments!'
          logger.tracec(self, '  Error: Insuficient arguments ' )
          return
        for o in args[1:]:
          delcmd = thecmd + o
          logger.tracec(self, '  running ' + delcmd)
          print 'Deleting bucket ' + o + ' ... '
          f = os.popen(delcmd)
          print '%s\n' % (f.read())
            
            






    




from threading import Thread
from time import gmtime, localtime, strftime
import time
import sys
import os
#from boto3 import *

# directory containing customed modules:
#sys.path.insert(0,"/apps/lib/python")
import logger

COMMAND_SHELL = "python -m aws-cmd"
MAX_LAUNCH = 2





class Ec2Commands():
    
    def __init__(self):
        logger.tracec(self,'Ec2Commands()')

    def process (self,cmd="", args = [], tags='', display_tags='', debug=False, ami=''):
        logger.tracec(self,'process(' + cmd + ')')
        if cmd == "ls":
          self.ls(args,tags,display_tags, debug)  
        if cmd == "start":
          self.start(args,tags)  
        if cmd == "stop":
          self.stop(args,tags)  
        if cmd == "terminate":
          self.terminate(args,tags)  
        if cmd == "launch":
          self.launch(args,tags,ami)  
      
    def ls (self,args,tags, display_tags,debug=False):
        logger.tracec(self, '  ls()')
        if display_tags:
          _dtags = '--ft=' + display_tags
        else:
          _dtags = ''
        if tags:
            _tags = '-t ' + tags
        else:
            _tags = ''
        thecmd = COMMAND_SHELL + ' ' + _tags + ' ' + _dtags
        if 'ip' in args:
          thecmd = thecmd + ' -I'
        if debug:
          print 'Command: ' + thecmd
        logger.tracec(self, '  running ' + thecmd)
        f = os.popen(thecmd)
        print '%s\n' % (f.read())

    # Don't allow bulk stop, too risky in production environment!!
    def start (self,args,tags):
        logger.tracec(self, '  start()')
        _tags = ''
#        if tags:
#            _tags = '-t ' + tags
#        else:
#            _tags = ''
        thecmd = COMMAND_SHELL + '  -C start ' + _tags
        for arg  in args[1:]:
          thecmd = thecmd + ' -i ' + arg
        logger.tracec(self, '  running ' + thecmd)
        f = os.popen(thecmd)
        print '%s\n' % (f.read())

    # Don't allow bulk stop, too risky in production environment!!
    def stop (self,args,tags):
        logger.tracec(self, '  stop()')
        _tags = ''
#        if tags:
#            _tags = '-t ' + tags
#        else:
#            _tags = ''
        thecmd = COMMAND_SHELL + ' -C stop ' + _tags
        for arg  in args[1:]:
          thecmd = thecmd + ' -i ' + arg
        logger.tracec(self, '  running ' + thecmd)
        f = os.popen(thecmd)
        print '%s\n' % (f.read())

    # Don't allow bulk terminate, too risky in production environment!!
    def terminate (self,args,tags):
        logger.tracec(self, '  terminate()')
        _tags = ''
#        if tags:
#            _tags = '-t ' + tags
#        else:
#            _tags = ''
        thecmd = COMMAND_SHELL + ' -C terminate ' + _tags
        for arg  in args[1:]:
          thecmd = thecmd + ' -i ' + arg
        logger.tracec(self, '  running ' + thecmd)
        f = os.popen(thecmd)
        print '%s\n' % (f.read())


    # Don't allow bulk launch, too risky in production environment!!
    def launch (self,args,tags,ami=''):
        logger.tracec(self, '  launch()')
        _tags = ''
#        if tags:
#            _tags = '-t ' + tags
#        else:
#            _tags = ''
        _errmsg = ''
        thecmd = COMMAND_SHELL + ' -C launch ' + _tags
        for arg  in args[1:]:
          if int(arg) > MAX_LAUNCH: 
            _errmsg = 'Launching >= ' + str(MAX_LAUNCH) + ' instances not allowed'
            print '%s\n' % (_errmsg)
            return
          thecmd = thecmd + ' -q ' + arg
        if ami != '':
          thecmd = thecmd + ' -a ' + ami
        logger.tracec(self, '  running ' + thecmd)
        f = os.popen(thecmd)
        print '%s\n' % (f.read())





    

